#  Copyright 2022 Google LLC
#  Copyright 2022 EPAM Systems
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import logging
import os
from typing import Tuple
from osdu_api.providers.blob_storage import get_client
from osdu_api.providers.types import FileLikeObject

from bundles.storage import BundleStorageClient

logger = logging.getLogger(__name__)


class FileNotFound(Exception):
    def __init__(self, filename: str):
        self.message = f"ERROR: File {filename} was not found."
        super().__init__(self.message)


class MinIOBundleStorageClient(BundleStorageClient):
    def __init__(self) -> None:
        self._client = get_client()
        self._bucket_name = os.environ["POLICY_BUCKET"]
        self._content_type = "application/x-gtar"

    def _get_bucket_uri(self, filename: str) -> str:
        return f"s3://{self._bucket_name}/{filename}"

    def download_file(
        self, filename: str, file: FileLikeObject
    ) -> Tuple[FileLikeObject, str]:
        try:
            uri = self._get_bucket_uri(filename)

            if self._does_file_exist(uri):
                self._client.download_to_file(uri, file)
                return file, uri
            else:
                raise FileNotFound(filename)

        except Exception as e:
            logger.error(f"Failed to download file from {uri} {e}")

    def upload_file(self, name: str, file: FileLikeObject) -> str:
        try:
            uri = self._get_bucket_uri(name)
            self._client.upload_file(uri, file, self._content_type)
            return uri
        except Exception as e:
            logger.error(f"Failed to upload file to {uri} {e}")

    def _does_file_exist(self, uri: str) -> bool:
        """Verify if a file exists in the given URI.

        :param uri: The AWS URI of the file.
        :type uri: str
        :return: A boolean indicating if the file exists
        :rtype: bool
        """
        return self._client.does_file_exist(uri)
