#!/usr/bin/env python3
import typer
import time
from datetime import datetime
import re


def main(
    path: str = typer.Argument(envvar="CI_PROJECT_NAME"),
    branch: str = typer.Option(
        "dev", "--branch", help="Release", envvar="CI_COMMIT_REF_NAME"
    ),
    build: str = typer.Option(
        "0", "--build", help="Build ID", envvar="CI_PIPELINE_IID"
    ),
    commit_message: str = typer.Option("", envvar="CI_COMMIT_MESSAGE"),
    commit_timestamp: str = typer.Option(None, envvar="CI_COMMIT_TIMESTAMP"),
    commit_ref_slug: str = typer.Option("", envvar="CI_COMMIT_REF_SLUG}"),
    commit_id: str = typer.Option("", envvar="CI_COMMIT_SHA"),
    versionfile: str = typer.Option("VERSION", "--version-file", help="Version file"),
    pyfile: str = typer.Option("_version.py", "--pyfile", help="python file"),
):
    """
    Simple utility to get pipeline version information and create _version.py
    """

    with open(versionfile, "r") as f:
        (major, minor, patch) = f.read().strip().split(".")

    if branch == "master" or branch == "main":
        release = f"rc{patch}.dev{build}"
    elif branch.startswith("release"):
        release = f"{patch}"
    elif branch.startswith("trusted"):
        release = f"b{patch}.dev{build}"
    else:
        release = f"a{patch}.dev{build}"

    __version__ = f"{major}.{minor}.{release}"
    print(f"{__version__}")
    milestone = int(minor) - 3

    if not commit_timestamp:
        today = datetime.now()
        commit_timestamp = today.isoformat()

    with open(path + "/" + pyfile, "w") as f:
        f.write('__apiversion__ = "1.0.0"\n')
        f.write(f'__version__ = "{__version__}"\n')
        f.write(f'__milestone__ = "M{milestone}"\n')
        f.write(f'__branch__ = "{branch}"\n')
        f.write(f'__build__ = "{build}"\n')
        f.write(f'__release__ = "{release}"\n')
        f.write(f"__buildtime__ = {time.time()}\n")
        f.write(f'__commitid__ = "{commit_id}"\n')
        f.write(f'__commitmessage__ = """{commit_message}"""\n')
        f.write(f'__committimestamp__ = "{commit_timestamp}"\n')
        f.write(f'__commitrefslug__ = "{commit_ref_slug}"\n')


if __name__ == "__main__":
    typer.run(main)
